/***************************************************************************
                          bsdscan.cpp  -  description
                             -------------------
    begin                : Fri Nov 26 1999
    copyright            : (C) 1999 by fflewddur
    email                : fflewddur@linuxfreak.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bsdscan.h"

// function get_input()
// task: prompt user for target, ports, etc.
// input: none
// output: port_s, port_f, target_ip
void get_input(char *target_ip, int args[]);

// function help()
// task: display usage info
// input: none
// output: none
void help(void);

// function arg_use()
// task: control argument usage and check for illegal combinations
// input: argc, argv
// ouput: args[]
boolean arg_use(int argc, char *argv[], int args[]);

// function err_quit()
// task: print an error message and exit the program
// input: error_str
// output: none
void err_quit(char *error_str);

// these functions are defined in scan.cpp
void scan(char *target_ip, int args[], char *argv[]);


/* -------------------------------------- */

int main(int argc, char *argv[])
{
	struct hostent *host;
	char target_ip[MAX_ADDR_LEN], temp_ip[INET_ADDRSTRLEN];
	boolean interactive; // on TRUE we prompt for input
	int args[MAX_ARGS]; // if an arg is used, it will be set to TRUE, otherwise it's FALSE

	/* arg list
		0 - logging enabled
		1 - points to the argv[] string with the log file
		2 - wildcard detected
		3 - location in string of wildcard
		4 - use stealth mode
		5 - ports defined on command-line
		6 - lowest port
		7 - highest port
		8 - only scan common ports
		9 - length of delay between sent packets
	*/

	// set all of args[] elements' to 0
	for (int i = 0; i <= MAX_ARGS; i++)
		args[i] = 0;

	// check for proper usage
	interactive = arg_use(argc, argv, args);

	// determine whether an address was specified on the command line,
	// and if not, get one now
	if (interactive)
	{
		get_input(target_ip, args);

		// take the string from fgets() and remove the newline from it
		for (int i = 5; i <= MAX_ADDR_LEN; i++)
		{
			if ( target_ip[i] == '\0')
			{
				target_ip[i-1] = '\0';
				// translate a domain name to an ip address, and put it in
				// target_ip
				host = Gethostbyname(target_ip);
				Inet_ntop(AF_INET, host->h_addr_list[0], temp_ip, INET_ADDRSTRLEN);
				strncpy(target_ip, temp_ip, sizeof(target_ip));
				break;
			}
		}
	}
    else if (args[WILDCARD] == 0)
    {
		// if the wildcard wasn't used, copy the ip from the command line into
		// target_ip, but don't let it overflow the buffer, and make sure it
		// terminates
		strncpy(target_ip, argv[1], sizeof(target_ip) - 1);
		target_ip[sizeof(target_ip) - 1] = '\0';

		// now convert target_ip into an ip incase it was input as a domain
		// name, and put the result back into target_ip
		host = Gethostbyname(target_ip);
		Inet_ntop(AF_INET, host->h_addr_list[0], temp_ip, INET_ADDRSTRLEN);
		strncpy(target_ip, temp_ip, sizeof(target_ip));
	}
	else
	{
		// if a wildcard was specified, copy the first three octects of the ip
		// into target_ip (argv[1] was already trimmed down to three octects in
		// arg_use() )
		strncpy(target_ip, argv[1], sizeof(target_ip) - 1);
	}

	// new scanning code
	scan(target_ip, args, argv);

	return 0;
}

/* -------------------------------------- */

void get_input(char *target_ip, int args[])
{
	printf("\nTarget (a.b.c.d):\t");
	fgets(target_ip, 16, stdin);
	printf("Starting port:\t\t");
	cin >> args[LOWPORT];
	printf("Ending port:\t\t");
	cin >> args[HIGHPORT];

	args[PORT] = 1; // same as declaring -p on the command line

	return;
}

/* -------------------------------------- */

void help(void)
{
	printf("Usage: bsdscan [target machine] [options]\n\n");
	printf("Options:\n");
	printf("\t--common\t-c\t\tOnly scans common ports\n");
	printf("\t--delay [x]\t-d [x]\t\tWait [x] seconds between sending packets\n");
	printf("\t--help\t\t-h\t\tPrints this usage information\n");
	printf("\t--log [file]\t-l [file]\tLogs results to [file]\n");
	printf("\t--ports [x] [y]\t-p [x] [y]\tScans between ports [x] and [y]\n");
	printf("\t--windows\t-w\t\tScans for SMB servers\n");
	printf("\t--version\t-v\t\tDisplays version info\n");

	exit (EXIT_FAILURE);
}

/* -------------------------------------- */

boolean arg_use(int argc, char *argv[], int args[])
{
	boolean interactive = TRUE; // TRUE mean prompt for input, false means
                             // we'll extract it from argv[]
	// if run without arguments, we'll need to prompt for input
	if (argc == 1)
		return (interactive);

	// go through all the argv's and properly execute them
	for (int i = 1; i < argc; i++)
	{
		if (!strcmp(argv[i], "--version") || !strcmp(argv[i], "-v"))
		{
			cout << "\tBsdScan v" << VERSION << " by fflewddur <fflewddur@linuxfreak.com>\n\t";
			printf("Released %s under version 2 of the GPL\n", RELEASE_DATE);
			exit (EXIT_SUCCESS);
		}

		if (!strcmp(argv[i], "--help") || !strcmp(argv[i], "-h"))
			help();

		if (!strcmp(argv[i], "--log") || !strcmp(argv[i], "-l"))
		{
			args[LOG] = 1;
			args[LOGFILE] = ++i;
			continue;
		}

		// check to see if a port range was specified
		if (!strcmp(argv[i], "--ports") || !strcmp(argv[i], "-p"))
		{
			// make sure 'scan common ports' wasn't specifed as well
			if (args[COMMON] || args[WINDOW])
				err_quit("Improper argument usage");

			args[PORT] = 1; // true
			args[LOWPORT] = atoi(argv[++i]);
			args[HIGHPORT] = atoi(argv[++i]);
			continue;
		}

		// check for the 'scan common ports' option
		if (!strcmp(argv[i], "--common") || !strcmp(argv[i], "-c"))
		{
			// make sure ports weren't specified as well
			if (args[PORT] || args[WINDOW])
				err_quit("Improper argument usage");

			args[COMMON] = 1;
			continue;
		}

		// check for a user-defined delay between packets being sent
		if (!strcmp(argv[i], "--delay") || !strcmp(argv[i], "-d"))
		{
			int delay = atoi(argv[++i]);
			args[DELAY] = delay;

			continue;
		}

		// we'll assume that no '-' means it's an ip address; error checking
		// will be done with Inet_pton later
		if (strncmp(argv[i], "-", 1))
		{
			// incase we hit this code twice (shouldn't happen) we exit the second time
			if (!interactive)
			{
	            printf("\nUnknown option: %s\n", argv[i]);
				help();
			}
			// check for wildcards, ie. '*'
			for (int j = 0; argv[i][j] != '\0'; j++)
			{
				if (argv[i][j] == '*')
	            {
					args[WILDCARD] = 1; // true
					args[WILDCARD_LOC] = j;
					argv[i][j] = '\0';
	            }
			}

			interactive = FALSE;
			continue;
		}

		if (!strcmp(argv[i], "--windows") || !strcmp(argv[i], "-w"))
		{
			if (args[PORT] || args[COMMON])
				err_quit("Can't use -p or -c together with -w");

			args[WINDOW] = 1;
			args[PORT] = 1;
			args[LOWPORT] = 139;
			args[HIGHPORT] = 139;
			i++;
			continue;
		}

		printf("\nUnknown option: %s\n", argv[i]);
		help();
	}

	return (interactive); // FALSE means we'll use argv[1] as target_ip
}

/* -------------------------------------- */

void err_quit(char *error_str)
{
	printf("*** Error: %s ***\n", error_str);
	exit (EXIT_FAILURE);
}

/* -------------------------------------- */