/***************************************************************************
                          scan.cpp  -  description
                             -------------------
    begin                : Fri Nov 26 1999
    copyright            : (C) 1999 by fflewddur
    email                : fflewddur@linuxfreak.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bsdscan.h"
#include <time.h>
#include <fstream.h>

// task: perform the specified scan
void scan(char *target_ip, int args[], char *argv[]);

// task: make sure target is reachable
int probe(char *target_ip, int log, ofstream &fout);

// task: shuffle the order of the ports to be scanned
void shuffle(int port_array[], int first_port, int last_port, int common);

// function declarations from bsdscan.cpp
void err_quit(char *error_str);

/* --------------------------------------------- */

void scan(char *target_ip, int args[], char *argv[])
{
	struct SA_IN target;
	struct servent *service;
	int low_port, high_port, sockfd, result, total, array_size;
	char ip_str[4];
	ofstream fout(1);

 	// output goes to fout, which either points to stdout (1) or a logfile
	// if one was specified
	if (args[LOG])
	{
		fout.close();
		fout.open(argv[args[LOGFILE]]);
	}

	if (args[PORT]) // PORT is set to 1 in arg_use() if --port is specified
	{
		low_port = args[LOWPORT];
		high_port = args[HIGHPORT];
	}
	else
	{
		low_port = 1;
		high_port = 1024;
	}

	array_size = (high_port - low_port) + 1;
	int *port_array = new int[array_size];

	bzero(&target, sizeof(target)); // fill target address with 0's
	target.sin_family = AF_INET;
	total = 0; // total number of open ports, needed to display output correctly

	// initalize port_array to 0
	for (int j = 0; j < array_size; j++)
		port_array[j] = 0;

	// shuffle the port numbers in port_array
	shuffle(port_array, low_port, high_port, args[COMMON]);

	for (int i = 1; i < 256; i++)
	{
		if (args[WILDCARD])
		{
			memset(ip_str, sizeof(ip_str), '\0'); // make sure ip_str is empty
			snprintf(ip_str, sizeof(ip_str), "%d", i); // put i into a ip_str
			target_ip[args[WILDCARD_LOC]] = '\0'; // clear target_ip after 1st three ocets
			strncat(target_ip, ip_str, sizeof(target_ip)); // add 1-255 onto target_ip for a subnet scan
		}

		Inet_pton(AF_INET, target_ip, &target.sin_addr);

		fout << "\nOpen ports on " << target_ip << " between " << low_port << " and " << high_port << ":\n\t----------------\n";

		// first make sure the target is reachable
		if (probe(target_ip, args[LOG], fout))
		{
			if (args[WILDCARD])
				continue;
			else
				exit(EXIT_FAILURE);
		}

		fout << "\tPort\tService\n";

		// actually scanning loop
		for (int i = 0; i <= array_size; i++)
		{
			// set up the socket and port info
			sockfd = Socket(AF_INET, SOCK_STREAM, 0);
			target.sin_port = htons(port_array[i]);

			result = connect(sockfd, (const SA *)&target, sizeof(SA));

			if (!result)
			{
				service = getservbyport(htons(port_array[i]), "tcp");
				if (service == NULL)
					fout << "\t" << port_array[i] << "\t" << "Unknown service\n";
				else
					fout << "\t" << port_array[i] << "\t" << service->s_name << "\n";
			}

			close (sockfd); // very important

			usleep((unsigned long)args[DELAY] * 1000000);
}

		if (!args[WILDCARD]) // if we're only scanning one machine, exit the loop
			break;
	}

	// let's be tidy now
	delete [] port_array;
	port_array = NULL;

	fout.close();

	return;
}

/* --------------------------------------------- */

int probe(char *target_ip, int log, ofstream &fout) // try to connect to target_ip, and if there is no route, print an error and return -1
{
	struct SA_IN target;
	int sockfd, result;

	bzero(&target, sizeof(target));
	target.sin_family = AF_INET;
	target.sin_port = htons(7);
	Inet_pton(AF_INET, target_ip, &target.sin_addr);

	sockfd = Socket(AF_INET, SOCK_STREAM, 0);
	result = connect(sockfd, (const SA *)&target, sizeof(struct SA));
	if (errno == ETIMEDOUT || errno == EHOSTUNREACH || errno == ENETUNREACH)
	{
		if (log)
		{
			if (errno == ETIMEDOUT)
				fout << "Connection timed out\n";
			else
				fout << "Network unreachable\n";
		}
		else
			perror("connect");

		close(sockfd);

		return -1;
	}

	close(sockfd);

	return 0;
}

/* --------------------------------------------- */

void shuffle(int port_array[], int low_port, int high_port, int common)
{
	int rnd_port, array_size;

	array_size = (high_port - low_port) + 1;

	// if the 'common' option was selected, only scan the following ports
	if (common)
	{
		int common_ports[] = {7, 13, 19, 21, 22, 23, 25, 37, 42, 49, 53, 69, 70, 79, 80, 88, 110, 111, 113, 119, 137, 139, 143, 512, 513, 1080, -1};
		for (unsigned int i = 0; i < sizeof(common_ports) / sizeof(int); i++)
			port_array[i] = common_ports[i];
	}
	// if 'common' wasn't specified, generate a random array of the ports to scan
	else
	{
		// setup an array containing the ports to be put into the final array
		int *ordered_array = new int[array_size];

		for (int i = 0; i < array_size; i++)
		{
			//printf("top of loop %d\n", i);
			ordered_array[i] = low_port + i;
			//printf("bottom of loop %d\n", i);
		}

		srandom(time(NULL)); // seed random() with the current time

		// run through this loop until every element in port_array is filled with ports in random order
		for (int i = 0; i < array_size; )
		{
			rnd_port = random() % array_size;
			if (ordered_array[rnd_port])
			{
				port_array[i++] = ordered_array[rnd_port]; // i is incremented here to avoid skipping over ports
				ordered_array[rnd_port] = 0;
			}
		}

		delete [] ordered_array;
		ordered_array = NULL;
	}
}

/* --------------------------------------------- */