/***************************************************************************
                          wrapper.cpp  -  description
                             -------------------
    begin                : Fri Nov 26 1999
    copyright            : (C) 1999 by fflewddur
    email                : fflewddur@linuxfreak.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "wrapper.h"

/* ******************************************************** */

int Socket(int family, int type, int protocol)
{
	int sockfd;

	if ( (sockfd = socket(family, type, protocol)) == -1)
	{
		perror("socket");
		exit(EXIT_FAILURE);
	}

	return sockfd;
}

/* ******************************************************** */

int Connect(int sockfd, const struct SA * addr, socklen_t addrlen)
{
	if ( connect(sockfd, addr, addrlen) != -1 )
		return 0;

	perror("connect");
	exit (EXIT_FAILURE);
}

/* ******************************************************** */

int Bind(int sockfd, const struct SA * addr, socklen_t addrlen)
{
	if ( bind(sockfd, addr, addrlen) != -1 )
		return 0;

	perror("bind");
	exit (EXIT_FAILURE);
}

/* ******************************************************** */

int Listen(int sockfd, int backlog)
{
	if ( listen(sockfd, backlog) != -1 )
		return 0;

	perror("listen");
	exit (EXIT_FAILURE);
}

/* ******************************************************** */

int Accept(int sockfd, struct SA *addr, socklen_t *addrlen)
{
	int connfd;
	if ( (connfd = accept(sockfd, addr, addrlen)) != -1 )
		return connfd;

	perror("accept");
	exit (EXIT_FAILURE);
}

/* ******************************************************** */

int Inet_pton(int family, const char *string, void *addr)
{
	switch ( inet_pton(family, string, addr) )
	{
		case -1:
		{
			perror("inet_pton");
			exit (EXIT_FAILURE);
		}
		case 0:
		{
			perror("inet_pton");
			exit (EXIT_FAILURE);
		}
		default:
			return 1;
	}
}

/* ******************************************************** */

const char * Inet_ntop(int family, const void *addr, char *string, size_t
len)
{
	if ( (inet_ntop(family, addr, string, len)) == NULL )
	{
		perror("inet_ntop");
		exit (EXIT_FAILURE);
	}

	return string;
}

/* ******************************************************** */

Sigfunc * signal(int signo, Sigfunc *func)
{
   struct sigaction act, oact;
   act.sa_handler = func;
   sigemptyset(&act.sa_mask);
   act.sa_flags = 0;
   if (signo == SIGALRM)
   {
      #ifdef SA_INTERRUPT
         act.sa_flags |= SA_INTERRUPT; /* sunos 4.x */
      #endif
   }
   else
   {
      #ifdef SA_RESTART
         act.sa_flags |= SA_RESTART; /* svr4, 4.4bsd */
      #endif
   }
   if (sigaction(signo, &act, &oact) < 0)
      return (SIG_ERR);
   return (oact.sa_handler);
}

/* ******************************************************** */

void sig_chld(int signo)
{
   pid_t pid;
   int stat;

   pid = wait(&stat);
   printf("child %d terminated\n", pid);
   return;
}

/* ******************************************************** */

struct hostent *Gethostbyname(const char *hostname)
{
   struct hostent *host;

   if ( (host = gethostbyname(hostname)) == NULL)
   {
      printf("%s\n", hstrerror(h_errno));
      exit(EXIT_FAILURE);
   }

   return host;
}

/* ******************************************************** */

struct servent *Getservbyport(int port, char *protocol)
{
	struct servent *service;

	if ( (service = getservbyport(port, protocol)) == NULL)
	{
		printf("%s\n", hstrerror(h_errno));
		exit(EXIT_FAILURE);
	}

	return service;
}

/* ******************************************************** */

int Setsockopt(int sockfd, int level, int optname, const void *optval, socklen_t optlen)
{
	if ( (setsockopt(sockfd, level, optname, optval, optlen)) < 0)
	{
		perror("setsockopt");
		exit(EXIT_FAILURE);
	}

	return 0;
}

/* ******************************************************** */

/* does not work */
ssize_t Sendto(int sockfd, const void *buffer, size_t nbytes, int flags, const struct SA *to, socklen_t addrlen)
{
	ssize_t bytes;

	if ( (bytes = sendto(sockfd, buffer, nbytes, flags, to, addrlen)) < 0)
	{
		perror("sendto");
		exit(EXIT_FAILURE);
	}

	return bytes;
}

/* ******************************************************** */

ssize_t Recvfrom(int sockfd, void *buffer, size_t nbytes, int flags, struct SA *from, socklen_t *addrlen)
{
	ssize_t bytes;

	if ( (bytes = recvfrom(sockfd, buffer, nbytes, flags, from, addrlen)) < 0)
	{
		perror("recvfrom");
		exit(EXIT_FAILURE);
	}

	return bytes;
}

/* ******************************************************** */
